"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JwsService = void 0;
const error_1 = require("../error");
const keyDidMapping_1 = require("../modules/dids/domain/key-type/keyDidMapping");
const DidKey_1 = require("../modules/dids/methods/key/DidKey");
const DidResolverService_1 = require("../modules/dids/services/DidResolverService");
const plugins_1 = require("../plugins");
const utils_1 = require("../utils");
const error_2 = require("../wallet/error");
const jwk_1 = require("./jose/jwk");
let JwsService = class JwsService {
    async createJwsBase(agentContext, options) {
        const { jwk, alg } = options.protectedHeaderOptions;
        const keyJwk = (0, jwk_1.getJwkFromKey)(options.key);
        // Make sure the options.key and jwk from protectedHeader are the same.
        if (jwk && (jwk.key.keyType !== options.key.keyType || !jwk.key.publicKey.equals(options.key.publicKey))) {
            throw new error_1.AriesFrameworkError(`Protected header JWK does not match key for signing.`);
        }
        // Validate the options.key used for signing against the jws options
        // We use keyJwk instead of jwk, as the user could also use kid instead of jwk
        if (keyJwk && !keyJwk.supportsSignatureAlgorithm(alg)) {
            throw new error_1.AriesFrameworkError(`alg '${alg}' is not a valid JWA signature algorithm for this jwk. Supported algorithms are ${keyJwk.supportedSignatureAlgorithms.join(', ')}`);
        }
        const base64Payload = utils_1.TypedArrayEncoder.toBase64URL(options.payload);
        const base64UrlProtectedHeader = utils_1.JsonEncoder.toBase64URL(this.buildProtected(options.protectedHeaderOptions));
        const signature = utils_1.TypedArrayEncoder.toBase64URL(await agentContext.wallet.sign({
            data: utils_1.TypedArrayEncoder.fromString(`${base64UrlProtectedHeader}.${base64Payload}`),
            key: options.key,
        }));
        return {
            base64Payload,
            base64UrlProtectedHeader,
            signature,
        };
    }
    async createJws(agentContext, { payload, key, header, protectedHeaderOptions }) {
        const { base64UrlProtectedHeader, signature } = await this.createJwsBase(agentContext, {
            payload,
            key,
            protectedHeaderOptions,
        });
        return {
            protected: base64UrlProtectedHeader,
            signature,
            header,
        };
    }
    /**
     *  @see {@link https://www.rfc-editor.org/rfc/rfc7515#section-3.1}
     * */
    async createJwsCompact(agentContext, { payload, key, protectedHeaderOptions }) {
        const { base64Payload, base64UrlProtectedHeader, signature } = await this.createJwsBase(agentContext, {
            payload,
            key,
            protectedHeaderOptions,
        });
        return `${base64UrlProtectedHeader}.${base64Payload}.${signature}`;
    }
    /**
     * Verify a JWS
     */
    async verifyJws(agentContext, { jws, payload }) {
        const base64Payload = utils_1.TypedArrayEncoder.toBase64URL(payload);
        const signatures = 'signatures' in jws ? jws.signatures : [jws];
        if (signatures.length === 0) {
            throw new error_1.AriesFrameworkError('Unable to verify JWS: No entries in JWS signatures array.');
        }
        const signerKeys = [];
        for (const jws of signatures) {
            const protectedJson = utils_1.JsonEncoder.fromBase64(jws.protected);
            const jwk = await this.jwkFromProtectedHeader(agentContext, protectedJson);
            if (!jwk.supportsSignatureAlgorithm(protectedJson.alg)) {
                throw new error_1.AriesFrameworkError(`alg '${protectedJson.alg}' is not a valid JWA signature algorithm for this jwk. Supported algorithms are ${jwk.supportedSignatureAlgorithms.join(', ')}`);
            }
            const data = utils_1.TypedArrayEncoder.fromString(`${jws.protected}.${base64Payload}`);
            const signature = utils_1.TypedArrayEncoder.fromBase64(jws.signature);
            signerKeys.push(jwk.key);
            try {
                const isValid = await agentContext.wallet.verify({ key: jwk.key, data, signature });
                if (!isValid) {
                    return {
                        isValid: false,
                        signerKeys: [],
                    };
                }
            }
            catch (error) {
                // WalletError probably means signature verification failed. Would be useful to add
                // more specific error type in wallet.verify method
                if (error instanceof error_2.WalletError) {
                    return {
                        isValid: false,
                        signerKeys: [],
                    };
                }
                throw error;
            }
        }
        return { isValid: true, signerKeys };
    }
    buildProtected(options) {
        var _a;
        if (!options.jwk && !options.kid) {
            throw new error_1.AriesFrameworkError('Both JWK and kid are undefined. Please provide one or the other.');
        }
        if (options.jwk && options.kid) {
            throw new error_1.AriesFrameworkError('Both JWK and kid are provided. Please only provide one of the two.');
        }
        return Object.assign(Object.assign({}, options), { alg: options.alg, jwk: (_a = options.jwk) === null || _a === void 0 ? void 0 : _a.toJson(), kid: options.kid });
    }
    async jwkFromProtectedHeader(agentContext, protectedHeader) {
        if (protectedHeader.jwk && protectedHeader.kid) {
            throw new error_1.AriesFrameworkError('Both JWK and kid are defined in the protected header. Only one of the two is allowed.');
        }
        // Jwk
        if (protectedHeader.jwk) {
            return (0, jwk_1.getJwkFromJson)(protectedHeader.jwk);
        }
        // Kid
        if (protectedHeader.kid) {
            if (!(0, utils_1.isDid)(protectedHeader.kid)) {
                throw new error_1.AriesFrameworkError(`Only DIDs are supported as the 'kid' parameter for JWS. '${protectedHeader.kid}' is not a did.`);
            }
            const didResolver = agentContext.dependencyManager.resolve(DidResolverService_1.DidResolverService);
            const didDocument = await didResolver.resolveDidDocument(agentContext, protectedHeader.kid);
            // This is a special case for Aries RFC 0017 signed attachments. It allows a did:key without a keyId to be used kid
            // https://github.com/hyperledger/aries-rfcs/blob/main/concepts/0017-attachments/README.md#signing-attachments
            if ((0, utils_1.isDid)(protectedHeader.kid, 'key') && !protectedHeader.kid.includes('#')) {
                return (0, jwk_1.getJwkFromKey)(DidKey_1.DidKey.fromDid(protectedHeader.kid).key);
            }
            return (0, jwk_1.getJwkFromKey)((0, keyDidMapping_1.getKeyFromVerificationMethod)(didDocument.dereferenceKey(protectedHeader.kid, ['authentication'])));
        }
        throw new error_1.AriesFrameworkError('Both JWK and kid are undefined. Protected header must contain one of the two.');
    }
};
JwsService = __decorate([
    (0, plugins_1.injectable)()
], JwsService);
exports.JwsService = JwsService;
//# sourceMappingURL=JwsService.js.map